{***********************************************************************}
{               HSoftware Components Collection                         }
{                                                                       }
{               Copyright (C) 1996 by Artem A. Berman                   }
{               E-mail: art@aber.kherson.ua                             }
{***********************************************************************}
{ This components can be freely used and distributed in commercial and  }
{ private environments, provied this notice is not modified in any way. }
{ ----------------------------------------------------------------------}
{ Version 1.0. Date last modified:  05/03/96                            }
{ ----------------------------------------------------------------------}

{$S-,I-,D-,L-}
unit AdvGrid;

interface

uses
   WinTypes, SysUtils, Messages, Classes, Controls, Grids;

type
  EAdvGridError = class(Exception);

  {TAdvStringGrid}
  TAdvStringGrid = class(TStringGrid)
  public
    procedure RemoveRows(RowIndex, RCount: LongInt);
    procedure InsertRows(RowIndex, RCount: LongInt);
    procedure RemoveCols(ColIndex, CCount: LongInt);
    procedure InsertCols(ColIndex, CCount: LongInt);
    procedure Clear;
    function isCell(SubStr: String; var ACol, ARow: LongInt): Boolean;
    procedure SaveToFile(FileName: String);
    procedure LoadFromFile(FileName: String);
    function CellToReal(ACol, ARow: LongInt): Real;
  end;

procedure Register;

implementation

{TAdvStringGrid.RemoveRows}
procedure TAdvStringGrid.RemoveRows(RowIndex, RCount : LongInt);
var
  i: LongInt;
begin
  for i := RowIndex to RowCount - 1 do Rows[i] := Rows[i + RCount];
  RowCount := RowCount - RCount;
end;

{TAdvStringGrid.InsertRows}
procedure TAdvStringGrid.InsertRows(RowIndex, RCount : LongInt);
var
  i: LongInt;
begin
  RowCount := RowCount + RCount;
  for i := RowCount - 1 downto RowIndex do Rows[i] := Rows[i - RCount];
end;

{TAdvStringGrid.RemoveCols}
procedure TAdvStringGrid.RemoveCols(ColIndex, CCount : LongInt);
var
  i: LongInt;
begin
  for i := ColIndex to ColCount - 1 do Cols[i] := Cols[i + CCount];
  ColCount := ColCount - CCount;
end;

{TAdvStringGrid.InsertCols}
procedure TAdvStringGrid.InsertCols(ColIndex, CCount : LongInt);
var
  i: LongInt;
begin
  ColCount := ColCount + CCount;
  for i := ColCount - 1 downto ColIndex do Cols[i] := Cols[i - CCount];
end;

{TAdvStringGrid.Clear}
procedure TAdvStringGrid.Clear;
var
  i: LongInt;
begin
  for i:= 0 to ColCount - 1 do Cols[i].Clear;
end;

{TAdvStringGrid.isCell}
function TAdvStringGrid.isCell(SubStr: String; var ACol, ARow: LongInt): Boolean;
var
  i, j: LongInt;
begin
  for i := 0 to RowCount - 1 do
  begin
    for j := 0 to ColCount - 1 do
    begin
      if Rows[i].Strings[j] = SubStr then
      begin
        ARow := i;
        ACol := j;
        Result := True;
        exit;
      end;
    end;
  end;
  Result := False;
end;

{TAdvStringGrid.SaveToFile}
procedure TAdvStringGrid.SaveToFile(FileName: String);
var
  i, j: LongInt;
  ss: string;
  f: TextFile;
begin
  AssignFile(f, FileName);
  Rewrite(f);
  ss := IntToStr(ColCount) + ',' + IntToStr(RowCount);
  Writeln(f, ss);

  for i := 0 to RowCount - 1 do
  begin
    for j := 0 to ColCount - 1 do
    begin
      if Cells[j, i] <> '' then
      begin
        ss := IntToStr(j) + ',' + IntToStr(i) + ',' + Cells[j, i];
        Writeln(f, ss);
      end;
    end;
  end;
  CloseFile(f);
end;

{TAdvStringGrid.LoadFromFile}
procedure TAdvStringGrid.LoadFromFile(FileName: String);
var
  X, Y: Integer;
  ss, ss1: string;
  f: TextFile;
begin
  AssignFile(f, FileName);
  Reset(f);
  if IOResult <> 0 then raise EAdvGridError.Create('File ' + FileName + ' not found');

  Readln(f, ss);
  if ss <> '' then
  begin
    ss1 := Copy(ss, 1, Pos(',', ss) - 1);
    ColCount := StrToInt(ss1);
    ss1 := Copy(ss, Pos(',', ss) + 1, Length(ss));
    RowCount := StrToInt(ss1);
  end;

  while not Eof(f) do
  begin
    Readln(f, ss);

    ss1 := Copy(ss, 1, Pos(',', ss) - 1);
    ss := Copy(ss, Pos(',', ss) + 1, Length(ss));
    X := StrToInt(ss1);


    ss1 := Copy(ss, 1, Pos(',', ss) - 1);
    ss := Copy(ss, Pos(',', ss) + 1, Length(ss));
    Y := StrToInt(ss1);

    Cells[X, Y] := ss;
  end;
  CloseFile(f);
end;

{TAdvStringGrid.CellToReal}
function TAdvStringGrid.CellToReal(ACol, ARow: LongInt): Real;
var
  i: Real;
  Code: Integer;
begin
  if Cells[ACol, ARow] <> '' then
  begin
    Val(Cells[ACol, ARow], i, Code);
    if Code <> 0 then raise
          EAdvGridError.Create('Error at position: ' +
          IntToStr(Code) + ' in Cell [' + IntToStr(ACol) + ', ' +
          IntToStr(ARow) + '].')
    else
    Result := i;
  end;
end;

{Register}
procedure Register;
begin
  RegisterComponents('Samples', [TAdvStringGrid]);
end;


end.
